import { Get, Injectable } from '@nestjs/common'

import { AbstractKeysProvider } from './abstract.provider.js'
import type { AdditionalData } from './abstract.provider.js'
import { proxyGetOp, proxyIndexOp } from './utils.js'
import { Indexes } from '../indexes.js'

@Injectable()
export class OpenPGPKeysProvider implements AbstractKeysProvider {
    readonly url = Object.freeze('https://keys.openpgp.org')

    @Get()
    async get(search: string, data: AdditionalData): Promise<string | 404> {
        return proxyGetOp(this.url + '/pks/lookup', search, data)
    }

    @Get()
    async index(search: string, data: AdditionalData): Promise<Indexes> {
        return proxyIndexOp(this.url + '/pks/lookup', search, data)
    }
}
