import type { KyResponse, ResponsePromise } from 'ky'
import ky from 'ky'
import type {
    AdditionalData,
    GetOperationReturn,
    HKPOperation,
} from './abstract.provider.js'
import type { Indexes } from '../indexes.js'
import { parseIndexes } from '../indexes.js'

const BEGIN_HEADER = '-----BEGIN PGP PUBLIC KEY BLOCK-----'
const END_HEADER = '-----END PGP PUBLIC KEY BLOCK-----'

export function getKey(raw: string): string {
    return raw
        .replace(new RegExp(`.*${BEGIN_HEADER}`, 'gm'), BEGIN_HEADER)
        .replace(new RegExp(`${END_HEADER}.*`, 'gm'), END_HEADER)
}

/**
 * Proxy request to a 3rd party keyserver via ky
 */
export function proxyRequest(
    url: string,
    op: HKPOperation,
    search: string,
    data: AdditionalData
): ResponsePromise {
    return ky(url, {
        headers: {
            'User-Agent': 'Cupid (https://cupid.blek.codes)',
        },
        timeout: 15000,
        searchParams: {
            op,
            search,
        },
    })
}

export async function proxyGetOp(
    url: string,
    search: string,
    data: AdditionalData
): Promise<GetOperationReturn> {
    let httpRes: KyResponse
    try {
        httpRes = await proxyRequest(url, 'index', search, data)
        if (httpRes.status !== 200) {
            return 404
        }
    } catch {
        return 404
    }

    return getKey(await httpRes.text())
}

export async function proxyIndexOp(
    url: string,
    search: string,
    data: AdditionalData
): Promise<Indexes> {
    let httpRes: KyResponse
    try {
        httpRes = await proxyRequest(url, 'index', search, data)
        if (httpRes.status !== 200) {
            return []
        }
    } catch {
        return []
    }

    const rawIndexes = await httpRes.text()
    return parseIndexes(rawIndexes.split('\n'))
}
